﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.OleDb;
using System.Configuration;

namespace OleSampleVs2010
{
    public partial class Form1 : Form
    {
        private readonly string CONNECTION_STRING;

        public Form1()
        {
            // pročitamo connection string iz App.config fajla
            this.CONNECTION_STRING = ConfigurationManager.ConnectionStrings["AccessDbConnectionString"].ConnectionString;

            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                ReloadDropdowns();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Dogodila se greška: " + ex.Message);
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                Save();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Dogodila se greška: " + ex.Message);
            }
        }

        #region DB access

        private void ReloadDropdowns()
        {
            cbKorisnici.Items.Clear();
            cbDogadjaji.Items.Clear();

            // inicijaliziramo konekciju na bazu
            using (OleDbConnection connection = new OleDbConnection(CONNECTION_STRING))
            {
                // otvorimo konekciju prema bazi
                connection.Open();

                // definiramo select query
                const string getKorisniciQuery = "select ID, korisnik from korisnici";
                // inicijaliziramo komandu sa select query-em
                using (OleDbCommand command = new OleDbCommand(getKorisniciQuery, connection))
                {
                    // pokrećemo ExecuteReader
                    using (OleDbDataReader reader = command.ExecuteReader())
                    {
                        // Čitamo rezultate dok ih ima
                        while (reader.Read())
                        {
                            string korisnik = reader["korisnik"] as string;
                            if (korisnik != null)
                            {
                                cbKorisnici.Items.Add(korisnik);
                            }
                        }
                    }
                }

                // definiramo select query
                const string getDogadjajiQuery = "select ID, dogadjaj from dogadjaji";
                // inicijaliziramo komandu sa select query-em
                using (OleDbCommand command = new OleDbCommand(getDogadjajiQuery, connection))
                {
                    // pokrećemo ExecuteReader
                    using (OleDbDataReader reader = command.ExecuteReader())
                    {
                        // Čitamo rezultate dok ih ima
                        while (reader.Read())
                        {
                            string dogadjaj = reader["dogadjaj"] as string;
                            if (dogadjaj != null)
                            {
                                cbDogadjaji.Items.Add(dogadjaj);
                            }
                        }
                    }
                }
            }
        }

        private void Save()
        {
            int? pk = null;
            // inicijaliziramo konekciju na bazu
            using (OleDbConnection connection = new OleDbConnection(CONNECTION_STRING))
            {
                // otvorimo konekciju prema bazi
                connection.Open();

                // definiramo insert query
                const string getKonfQuery = "insert into Tomislav(Korisnik, Dogadjaj, DatumOd, DatumDo) values (@Korisnik, @Dogadjaj, @DatumOd, @DatumDo)";

                // napunimo lisu parametara
                List<OleDbParameter> parameters = new List<OleDbParameter>();
                parameters.Add(GetParam("@Korisnik", this.cbKorisnici.SelectedItem ?? DBNull.Value, OleDbType.BSTR));
                parameters.Add(GetParam("@Dogadjaj", this.cbDogadjaji.SelectedItem ?? DBNull.Value, OleDbType.BSTR));
                parameters.Add(GetParam("@DatumOd", this.dtDatumOd.Value, OleDbType.Date));
                parameters.Add(GetParam("@DatumDo", this.dtDatumDo.Value, OleDbType.Date));

                int rowsAffected = 0;
                // inicijaliziramo komandu sa insert query-em
                using (OleDbCommand command = new OleDbCommand(getKonfQuery, connection))
                {
                    // dodajemo parametre
                    command.Parameters.AddRange(parameters.ToArray());

                    // izvršavamo komandu. Pošto insert query ne vraća ništa, executamo pomoću metode ExecuteNonQuery
                    rowsAffected = command.ExecuteNonQuery();
                }

                // Provjera je li zapis uspješno sjeu u bazu
                if (rowsAffected > 0)
                {
                    // Često nam treba primarni ključ zapisa koji smo spremili

                    // Definiramo query za dohvat ID-a zadnjeg insertanog zapisa
                    const string selectIdentityQuery = "select @@Identity";

                    // inicijaliziramo komandu prethodnim query-em
                    using (OleDbCommand command = new OleDbCommand(selectIdentityQuery, connection))
                    {
                        // spremamo skalarnu vrijednost koju kveri vrati u varijablu
                        pk = (int)command.ExecuteScalar();
                    }
                }
            }

            if (pk.HasValue)
            {
                MessageBox.Show(string.Format("Zapis je uspješno spremljen, id: {0}", pk));
            }
        }

        private OleDbParameter GetParam(string name, object value, OleDbType type)
        {
            OleDbParameter ret = new OleDbParameter(name, type);
            ret.Value = value;

            return ret;
        }

        #endregion
    }
}
